@@ -67,6 +67,7 @@ optParser desc = FsckOptions
 	<*> optional (parseRemoteOption $ strOption 
 		( long "from" <> short 'f' <> metavar paramRemote 
 		<> help "check remote"
+		<> completeRemotes
 		))
 	<*> optional parseincremental
 	<*> optional (parseKeyOptions False)
@@ -160,7 +161,7 @@ performRemote key file backend numcopies remote =
 			( return (Just True)
 			, ifM (Annex.getState Annex.fast)
 				( return Nothing
-				, Just <$>
+				, Just . fst <$>
 					Remote.retrieveKeyFile remote key Nothing tmp dummymeter
 				)
 			)
@@ -348,12 +349,15 @@ checkBackendOr :: (Key -> Annex String) -> Backend -> Key -> FilePath -> Annex B
 checkBackendOr bad backend key file =
 	checkBackendOr' bad backend key file (return True)
 
+-- The postcheck action is run after the content is verified,
+-- in order to detect situations where the file is changed while being
+-- verified (particularly in direct mode).
 checkBackendOr' :: (Key -> Annex String) -> Backend -> Key -> FilePath -> Annex Bool -> Annex Bool
 checkBackendOr' bad backend key file postcheck =
-	case Types.Backend.fsckKey backend of
+	case Types.Backend.verifyKeyContent backend of
 		Nothing -> return True
-		Just a -> do
-			ok <- a key file
+		Just verifier -> do
+			ok <- verifier key file
 			ifM postcheck
 				( do
 					unless ok $ do
