@@ -24,7 +24,6 @@ import Command
 import Utility.DataUnits
 import Utility.DiskFree
 import Annex.Content
-import Annex.Link
 import Types.Key
 import Logs.UUID
 import Logs.Trust
@@ -80,7 +79,7 @@ emptyStatInfo = StatInfo Nothing Nothing M.empty Nothing
 type StatState = StateT StatInfo Annex
 
 cmd :: Command
-cmd = noCommit $ dontCheck repoExists $ withGlobalOptions (jsonOption : annexedMatchingOptions) $
+cmd = noCommit $ withGlobalOptions (jsonOption : annexedMatchingOptions) $
 	command "info" SectionQuery
 		"shows  information about the specified item or the repository as a whole"
 		(paramRepeating paramItem) (seek <$$> optParser)
@@ -127,8 +126,7 @@ itemInfo o p = ifM (isdir p)
 				v' <- Remote.nameToUUID' p
 				case v' of
 					Right u -> uuidInfo o u
-					Left _ -> maybe noinfo (fileInfo o p)
-						=<< isAnnexLink p
+					Left _ -> ifAnnexed p (fileInfo o p) noinfo
 	)
   where
 	isdir = liftIO . catchBoolIO . (isDirectory <$$> getFileStatus)
